use crate::error::GeneralError;
use reqwest::header::{HeaderMap, AUTHORIZATION, ACCEPT};
use reqwest::{ClientBuilder, StatusCode};
use serde::de::DeserializeOwned;
use url::Url;
use crate::config::GiteaConfig;

#[derive(Debug)]
pub struct Client {
    token: String,
    base_uri: String,
}

impl Client {

    pub fn from_config(config: &GiteaConfig) -> Self {
        Self::new(&config.token, &config.domain)
    }

    pub fn new(token: &String, domain: &String) -> Self {
        Client {
            token: token.clone(),
            base_uri: format!("https://{}", domain.clone()),
        }
    }

    pub async fn get<T: DeserializeOwned>(&self, url: Url) -> Result<T, GeneralError> {
        let mut headers = HeaderMap::new();
        headers.append(AUTHORIZATION, self.token.parse()?);
        headers.append(ACCEPT, "application/json".parse()?);

        let client = ClientBuilder::new()
            .default_headers(headers)
            .build()
            .unwrap();

        let response = client.get(url.clone()).send().await?;

        match response.status() {
            StatusCode::OK => {
                let result: T = response.json().await?;

                return Ok(result);
            }
            _ => Err(GeneralError {
                description: format!(
                    "Could not call GET on {:?}, error code {}",
                    url,
                    response.status()
                ),
            }),
        }
    }
}
